#ifndef __ISEC_H
#define __ISEC_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "isec_def.h"      /* 基础结构定义 */
#include "isec_crypto.h"   /* 密码算法 */
#include "isec_wbsm4.h"    /* 白盒算法 */
#include "isec_ke.h"       /* 密钥协商 */

#define ISEC_VERSION "2.0.4.1"

/**
 * @brief 全局初始化
 *
 * 在一个进程启动后，需要且仅需要一次调用本接口，进行全局初始化后，才能使用其他接口。
 * 后续使用过程中，不再需要调用该接口进行初始化。
 *
 * @return int 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_global_init(void);

/**
 * @brief 全局设置日志
 *
 * 全局设置日志文件输出路径，如果不设置，则不输出日志。
 *
 * @param log_file  [in] 可空，日志文件路径（注：仅创建文件，不会创建中间目录）。
 *
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_global_set_logger(const char *log_file);

/**
 * @brief 全局初始化清理
 *
 * 在一个进程结束前，需要且仅需要一次调用本接口，进行全局初始化清理，以释放资源。
 *
 * 注：若应用在集成过程中，没有适当的时机进行释放，可以不调用本接口，但禁止频繁调用全局初始化清理和全局初始化。
 *
 * @return int 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_global_cleanup(void);

/**
 * @brief 获取版本号字符串
 *
 * 返回非空只读版本号字符串
 *
 * @return const char* 非空只读版本号字符串
 */
ISEC_API
const char *isec_version(void);

/**
 * @brief 获取错误码的字符串描述信息
 *
 * 返回只读错误码描述字符串。
 *
 * @param isec_err  [in] ISEC错误码
 * @return const char*  非空只读错误码对应的字符串描述
 */
ISEC_API
const char *isec_strerror(int isec_err);

/**
 * @brief 解析非对称密钥数据
 *
 * 自动解析PKCS1, PKCS8, PKCS12(PFX)格式的DER/Base64/PEM编码的二进制或字符串数据的私钥或公钥（例如："MII..."，"-----BEGIN ..."）
 * 若type设置为isec_keypair_type_sm2，则额外支持解析SM2裸密钥（若私钥，则至少需要传入私钥数据；若公钥，则至少需要传入公钥数据）：
 *   若密钥是二进制字节数据：
 *     若传入私钥，则私钥长度为32字节；
 *     若传入公钥，则长度为33/64/65字节；
 *   若密钥是十六进制字符串数据：
 *     若传入私钥，则私钥长度为64字节；
 *     若传入公钥，则长度为66/128/130字节；
 *   若密钥是Base64字节字符串：
 *     若传入私钥，则私钥长度大约为44字节；
 *     若传入公钥，则长度大约为44/88字节；
 * 若type设置为isec_keypair_type_rsa，则额外支持解析RSA裸密钥（若私钥，则必须同时传入等长的私钥和公钥数据；若公钥，则至少需要传入公钥数据）：
 *   仅支持传入二进制字节数据，不支持直接传入十六进制字符串和Base64字符串数据
 *
 * @param type      [in] 可空，非对称密钥类型
 * @param sk        [in] 可空，包含私钥的二进制字节数据或Base64编码字符串
 * @param sk_len    [in] 可空，包含私钥的数据的长度
 * @param pk        [in] 可空，包含公钥的二进制字节数据或Base64编码字符串
 * @param pk_len    [in] 可空，包含公钥的数据的长度
 * @param password  [in] 可空，保护私钥的口令
 * @param keypair   [out] 非对称密钥句柄，由isec_keypair_free进行释放内存
 *
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_keypair_unmarshal(isec_keypair_type_e type,
    const void *sk, int sk_len, const void *pk, int pk_len, const char *password,
    isec_keypair_t **keypair);

/**
 * @brief 提取非对称密钥数据
 *
 * 从非对称密钥句柄中，提取PKCS8格式的公钥或私钥数据。
 *
 * @param keypair  [in] 非空，非对称密钥句柄
 * @param format   [in] 非空，输出密钥的格式
 * @param sk       [out] 可空，私钥二进制字节数据
 * @param sk_len   [in] 可空，私钥缓冲区大小 [out] 私钥长度
 * @param pk       [out] 可空，公钥二进制字节数据
 * @param pk_len   [in] 可空，公钥缓冲区大小 [out] 公钥长度
 *
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_keypair_marshal(isec_keypair_t *keypair, isec_format_e format,
    void *sk, int *sk_len, void *pk, int *pk_len);

/**
 * @brief 释放非对称密钥对句柄
 *
 * 释放非对称密钥对句柄以释放内存。
 *
 * @param keypair  [in] 非空，非对称密钥句柄
 *
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_keypair_free(isec_keypair_t *keypair);

/**
 * @brief 解析证书数据
 *
 * 自动解析X509, PKCS7, PKCS12(PFX)格式的DER/Base64/PEM编码的证书。
 *
 * @param data      [in] 证书缓冲区
 * @param data_len  [in] 证书长度
 * @param password  [in] 保护口令
 * @param cert      [out] 证书句柄，由isec_certificate_free释放内存
 *
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_certificate_unmarshal(const void *data, int data_len, const char *password,
    isec_certificate_t **cert);

/**
 * @brief 提取证书信息
 *
 * 从证书句柄中，提取证书字段信息，例如序列号、主题、颁发者、有效期等。
 *
 * @param cert     [in] 证书句柄
 * @param info     [in] 证书信息类型
 * @param out      [out] 证书信息
 * @param out_len  [in] 缓冲区大小 [out] 证书信息长度
 *
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_certificate_parse(isec_certificate_t *cert, isec_certificate_info_e info,
    char *out, int *out_len);

/**
 * @brief 3.1.12 根证书校验
 *
 * 校验证书的有效期：
 *  若未生效则返回isec_err_cert_not_yet_valid；
 *  若已过期则返回isec_err_cert_expired；
 * 若设置根证书文件或根证书路径，则进行根证书校验：
 *  若签名验证失败则返回isec_err_verify_fail
 *
 * @param cert     [in] 非空，证书句柄
 * @param ca_file  [in] 可空，根证书文件路径
 * @param ca_path  [in] 可空，根证书目录路径（该目录下的根证书文件需要通过命令“c_rehash”处理过）
 *
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_certificate_verify(isec_certificate_t *cert, const char *ca_file, const char *ca_path);

/**
 * @brief 释放证书句柄
 *
 * @param cert  [in] 证书句柄
 *
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_certificate_free(isec_certificate_t *cert);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __ISEC_H */

