#ifndef __ISEC_CRYPTO_H
#define __ISEC_CRYPTO_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "isec.h"

/**
 * @brief HEX编码
 *
 * 功能：对输入的任意类型的数据，以字节方式读取并转换为HEX字符串数据。
 *
 * 注1：输入数据被作为二进制字节数据进行编码，内部不对输入数据做其他编码处理（例如中文编码）
 *
 * @param in       [in] 非空，待编码数据指针
 * @param in_len   [in] 非负数，待编码数据长度
 * @param out      [out] 可空，编码后字符串数据（传NULL时，由out_len返回缓冲区大小）
 * @param out_len  [in] 非空，缓冲区大小 [out] 非空，编码后字符串数据长度
 * @return int     [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_hex_encode(const void *in, int in_len, char *out, int *out_len);

/**
 * @brief HEX解码
 *
 * 功能：对输入的HEX字符串数据，进行HEX解码并按照二进制字节数据写入缓冲区，应用层自行传入指定类型指针。
 *
 * 注1：解码后输出数据按照二进制字节数据存储，内部不对输出数据做其他编码处理（例如中文编码）
 *
 * @param in       [in] 非空，编码数据
 * @param in_len   [in] 非负数，编码数据长度
 * @param out      [out] 可空，解码后数据（传NULL时，由out_len返回缓冲区大小）
 * @param out_len  [in] 非空，缓冲区大小 [out] 解码后数据字节长度
 * @return int     [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_hex_decode(const char *in, int in_len, void *out, int *out_len);

/**
 * @brief Base64编码
 *
 * 功能：对输入的任意类型的数据，进行Base64编码并输出字符串数据。
 *
 * 注1：输入数据被作为二进制字节数据进行编码，内部不对输入数据做其他编码处理（例如中文编码）
 *
 * @param in       [in] 非空，待编码数据指针
 * @param in_len   [in] 非负数，待编码数据长度
 * @param out      [out] 可空，编码后字符串数据（传NULL时，由out_len返回缓冲区大小）
 * @param out_len  [in] 非空，缓冲区大小 [out] 非空，编码后字符串数据长度
 * @return int     [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_base64_encode(const void *in, int in_len, char *out, int *out_len);

/**
 * @brief Base64解码
 *
 * 功能：对输入的字符串数据，进行Base64解码并按照二进制字节数据写入缓冲区，应用层自行传入指定类型指针。
 *
 * 注1：解码后输出数据按照二进制字节数据存储，内部不对输出数据做其他编码处理（例如中文编码）
 *
 * @param in       [in] 非空，编码数据
 * @param in_len   [in] 非负数，编码数据长度
 * @param out      [out] 可空，解码后数据（传NULL时，由out_len返回缓冲区大小）
 * @param out_len  [in] 非空，缓冲区大小 [out] 解码后数据字节长度
 * @return int     [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_base64_decode(const char *in, int in_len, void *out, int *out_len);

/**
 * @brief 产生随机数
 *
 * 功能：随机产生指定长度的二进制随机数字节数据。
 *
 * @param len   [in] 非空，指定产生的随机数长度
 * @param data  [out] 非空，随机数字节数据，缓冲区大小建议设置为[len]字节
 * @return int  [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_gen_random(int len, unsigned char *data);

/**
 * @brief 产生对称密钥
 *
 * 功能：根据指定的对称算法（alg），随机产生该算法指定长度的对称密钥字节数据。
 *
 * @param alg      [in] 非空，对称算法
 * @param key      [in] 可空，对称密钥字节数据（为NULL时由key_len返回缓冲区大小，建议设置为ISEC_MAX_KEY_LEN字节）
 * @param key_len  [in] 非空，缓冲区大小 [out] 非空，对称密钥字节长度
 * @return int     [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_gen_symmkey(isec_cipher_alg_e alg,
    unsigned char *key, int *key_len);

/**
 * @brief 产生非对称密钥对
 *
 * 功能：产生指定密钥类型（type）和密钥强度（strength）的非对称密钥对句柄。
 *
 * @param type      [in] 非空，密钥类型
 * @param strength  [in] 非空，密钥强度
 * @param keypair   [out] 非空，密钥对句柄，使用完成后由isec_keypair_free进行释放内存
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_gen_keypair(isec_keypair_type_e type, isec_keypair_strength_e strength,
    isec_keypair_t **keypair);

/**
 * @brief 产生证书请求
 *
 * 功能：根据证书请求配置（config），使用其指定密钥对（keypair）中的私钥和摘要算法（hash）
 * 产生指定主题（dn）的证书请求。
 *
 * 注1：密钥对（keypair）必须包含私钥
 *
 * @param config   [in] 非空，证书请求配置结构体
 * @param csr      [out] 可空，DER编码的证书请求二进制数据（传NULL时，由csr_len返回缓冲区大小）
 * @param csr_len  [in] 非空，证书请求缓冲区大小 [out] 非空，证书请求长度
 * @return int     [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_gen_csr(const isec_csr_config_t *config, unsigned char *csr, int *csr_len);

/**
 * @brief 密钥派生
 *
 * 场景：相同的任意长度的输入数据，可以产生相同的指定长度的输出数据
 *
 * 功能：使用指定原文数据派生出指定长度的密钥数据。
 *
 * @param hash      [in] 非空，摘要算法
 * @param data      [in] 非空，原文指针
 * @param data_len  [in] 非负数，原文字节长度
 * @param out_len   [in] 非负数，待派生密钥长度
 * @param out       [out] 非空，输出缓冲区，缓冲区大小建议设置为[out_len]字节
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_kdf(isec_hash_alg_e hash, const void *data, int data_len,
    int out_len, unsigned char *out);

/**
 * @brief 单组摘要
 *
 * 场景：应用可以一次性加载所有数据到内存中，直接对所有数据进行摘要计算
 *
 * 功能：若应用已加载完所有数据到内存，该接口对指定长度的字节数据消息进行密码杂凑计算，并输出摘要结果。
 *
 * 注1：待摘要数据被作为字节数据进行编码，内部不对待摘要数据做其他编码处理（例如中文编码）
 *
 * @param config    [in] 非空，摘要配置结构体
 * @param data      [in] 非空，待摘要数据
 * @param data_len  [in] 非负数，待摘要数据长度
 * @param md        [out] 可空，摘要缓冲区（传NULL时，由md_len返回缓冲区大小，建议设置为[ISEC_MAX_MD_LEN]字节）
 * @param md_len    [in] 非空，摘要缓冲区大小 [out] 非空，摘要长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_hash(const isec_hash_config_t *config,
    const void *data, int data_len, unsigned char *md, int *md_len);

/**
 * @brief 多组摘要初始化
 *
 * 场景：若应用需要对所有数据进行摘要计算，但每次只能加载部分数据到内存。
 *
 * 功能：初始化密码杂凑计算操作，指定计算密码杂凑的算法，返回摘要句柄。
 *
 * @param config  [in] 非空，摘要配置
 * @param hash    [out] 非空，摘要句柄
 * @return int    [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_hash_init(const isec_hash_config_t *config, isec_hash_t **hash);

/**
 * @brief 多组数据摘要
 *
 * 场景：若应用需要对所有数据进行摘要计算，但每次只能加载部分数据到内存。
 *
 * 功能：对多个分组的消息进行密码杂凑计算。
 *
 * 流程：
 * 1.首次调用本接口之前，必须调用isec_crypto_hash_init初始化密码杂凑计算操作；
 * 2.若应用数据加载未结束，应用将本次加载的任意长度数据作为输入，调用本接口进行计算，
 * 内部将缓存结果存入句柄，等待下一次数据加载并输入；
 * 3.若应用数据加载已结束，调用isec_crypto_hash_final输出摘要值；
 *
 * 注1：多组数据摘要效果等同于一次性加载所有数据并调用isec_crypto_hash接口。
 *
 * @param hash      [in] 非空，摘要句柄
 * @param data      [in] 非空，待摘要数据
 * @param data_len  [in] 非负数，待摘要数据长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_hash_update(isec_hash_t *hash, const void *data, int data_len);

/**
 * @brief 多组数据摘要结束
 *
 * 场景：若应用需要对所有数据进行摘要计算，但每次只能加载部分数据到内存。
 *
 * 功能：结束多个分组消息的密码杂凑计算操作，将密码杂凑结果保存到指定的缓冲区。
 *
 * 流程：完成后，调用isec_crypto_hash_clean关闭摘要句柄，释放内存。
 *
 * @param hash    [in] 非空，摘要句柄
 * @param md      [out] 可空，摘要缓冲区（传NULL时，由md_len返回缓冲区大小，建议设置为[ISEC_MAX_MD_LEN]字节）
 * @param md_len  [in] 非空，摘要缓冲区大小 [out] 非空，摘要长度
 * @return int    [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_hash_final(isec_hash_t *hash, unsigned char *md, int *md_len);

/**
 * @brief 关闭摘要句柄
 *
 * 功能：关闭isec_crypto_hash_init创建的句柄，释放内存。
 *
 * @param hash  [in] 非空，摘要句柄
 * @return int  [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_hash_clean(isec_hash_t *hash);

/**
 * @brief 单组数据HMAC消息鉴别码运算
 *
 * 场景：应用可以一次性加载所有数据到内存中，直接对所有数据进行HMAC计算
 *
 * 功能：计算单一分组数据的消息鉴别码
 *
 * 注1：待HMAC数据被作为字节数据进行编码，内部不对待HMAC数据做其他编码处理（例如中文编码）
 *
 * @param config    [in] 非空，消息鉴别码配置
 * @param data      [in] 非空，待计算数据缓冲区
 * @param data_len  [in] 非负数，待计算数据长度
 * @param md        [out] 可空，计算结果缓冲区（传NULL时，由md_len返回缓冲区大小，建议设置为[ISEC_MAX_MD_LEN]字节）
 * @param md_len    [in] 非空，计算结果缓冲区大小 [out] 非空，计算结果长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_hmac(const isec_hmac_config_t *config,
    const void *data, int data_len, unsigned char *md, int *md_len);

/**
 * @brief 多组数据HMAC消息鉴别码初始化
 *
 * 场景：若应用需要对所有数据进行HMAC计算，但每次只能加载部分数据到内存。
 *
 * 功能：初始化消息鉴别码计算操作，设置计算消息鉴别码的所需参数，并返回消息鉴别码句柄。
 *
 * @param config  [in] 非空，消息鉴别码配置
 * @param hmac    [out] 非空，消息鉴别码句柄
 * @return int    [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_hmac_init(const isec_hmac_config_t *config, isec_hmac_t **hmac);

/**
 * @brief 多组数据HMAC消息鉴别码运算
 *
 * 场景：若应用需要对所有数据进行HMAC计算，但每次只能加载部分数据到内存。
 *
 * 功能：计算多个分组数据的消息鉴别码。
 *
 * 流程：
 * 1.首次调用本接口之前，必须调用isec_crypto_hmac_init初始化消息鉴别码计算操作；
 * 2.若应用数据加载未结束，应用将本次加载的任意长度数据作为输入，调用本接口进行计算，
 * 内部将缓存结果存入句柄，等待下一次数据加载并输入；
 * 3.若应用数据加载已结束，调用isec_crypto_hmac_final输出摘要值；
 *
 * 注1：多组数据HMAC效果等同于一次性加载所有数据并调用isec_crypto_hmac接口。
 *
 * @param hmac      [in] 非空，消息鉴别码句柄
 * @param data      [in] 非空，待计算数据缓冲区
 * @param data_len  [in] 非负数，待计算数据长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_hmac_update(isec_hmac_t *hmac, const void *data, int data_len);

/**
 * @brief 多组数据HMAC消息鉴别码结束
 *
 * 场景：若应用需要对所有数据进行HMAC计算，但每次只能加载部分数据到内存。
 *
 * 功能：结束多个分组数据的消息鉴别码计算操作。
 *
 * 流程：完成后，调用isec_crypto_hmac_clean关闭消息鉴别码句柄，释放内存。
 *
 * @param hmac    [in] 非空，消息鉴别码句柄
 * @param md      [out] 可空，计算结果缓冲区（传NULL时，由md_len返回缓冲区大小，建议设置为[ISEC_MAX_MD_LEN]字节）
 * @param md_len  [in] 非空，计算结果缓冲区大小 [out] 非空，计算结果长度
 * @return int    [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_hmac_final(isec_hmac_t *hmac, unsigned char *md, int *md_len);

/**
 * @brief 关闭HMAC消息鉴别码句柄
 *
 * 功能：关闭isec_crypto_hmac_init创建的句柄，释放内存。
 *
 * @param hmac  [in] 非空，消息鉴别码句柄
 * @return int  [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_hmac_clean(isec_hmac_t *hmac);

/**
 * @brief 单组数据CMAC消息鉴别码运算
 *
 * 场景：应用可以一次性加载所有数据到内存中，直接对所有数据进行CMAC计算
 *
 * 功能：计算单一分组数据的CMAC消息鉴别码
 *
 * 注1：待CMAC数据被作为字节数据进行编码，内部不对待CMAC数据做其他编码处理（例如中文编码）
 *
 * @param config    [in] 非空，消息鉴别码配置
 * @param data      [in] 非空，待计算数据缓冲区
 * @param data_len  [in] 非负数，待计算数据长度
 * @param md        [out] 可空，计算结果缓冲区（传NULL时，由md_len返回缓冲区大小，建议设置为[ISEC_MAX_MD_LEN]字节）
 * @param md_len    [in] 非空，计算结果缓冲区大小 [out] 非空，计算结果长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_cmac(const isec_cmac_config_t *config, const void *data, int data_len, unsigned char *md, int *md_len);

/**
 * @brief 多组数据CMAC消息鉴别码初始化
 *
 * 场景：若应用需要对所有数据进行CMAC计算，但每次只能加载部分数据到内存。
 *
 * 功能：初始化消息鉴别码计算操作，设置计算消息鉴别码的所需参数，并返回消息鉴别码句柄。
 *
 * @param config  [in] 非空，消息鉴别码配置
 * @param cmac    [out] 非空，消息鉴别码句柄
 * @return int    [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_cmac_init(const isec_cmac_config_t *config, isec_cmac_t **cmac);

/**
 * @brief 多组数据CMAC消息鉴别码运算
 *
 * 场景：若应用需要对所有数据进行CMAC计算，但每次只能加载部分数据到内存。
 *
 * 功能：计算多个分组数据的消息鉴别码。
 *
 * 流程：
 * 1.首次调用本接口之前，必须调用isec_crypto_cmac_init初始化消息鉴别码计算操作；
 * 2.若应用数据加载未结束，应用将本次加载的任意长度数据作为输入，调用本接口进行计算，
 * 内部将缓存结果存入句柄，等待下一次数据加载并输入；
 * 3.若应用数据加载已结束，调用isec_crypto_cmac_final输出摘要值；
 *
 * 注1：多组数据HMAC效果等同于一次性加载所有数据并调用isec_crypto_cmac接口。
 *
 * @param cmac      [in] 非空，消息鉴别码句柄
 * @param data      [in] 非空，待计算数据缓冲区
 * @param data_len  [in] 非负数，待计算数据长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_cmac_update(isec_cmac_t *cmac, const void *data, int data_len);

/**
 * @brief 多组数据CMAC消息鉴别码结束
 *
 * 场景：若应用需要对所有数据进行CMAC计算，但每次只能加载部分数据到内存。
 *
 * 功能：结束多个分组数据的消息鉴别码计算操作。
 *
 * 流程：完成后，调用isec_crypto_cmac_clean关闭消息鉴别码句柄，释放内存。
 *
 * @param cmac    [in] 非空，消息鉴别码句柄
 * @param md      [out] 可空，计算结果缓冲区（传NULL时，由md_len返回缓冲区大小，建议设置为[ISEC_MAX_MD_LEN]字节）
 * @param md_len  [in] 非空，计算结果缓冲区大小 [out] 非空，计算结果长度
 * @return int    [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_cmac_final(isec_cmac_t *cmac, unsigned char *md, int *md_len);

/**
 * @brief 关闭CMAC消息鉴别码句柄
 *
 * 功能：关闭isec_crypto_cmac_init创建的句柄，释放内存。
 *
 * @param cmac  [in] 非空，消息鉴别码句柄
 * @return int  [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_cmac_clean(isec_cmac_t *cmac);

/**
 * @brief 单组数据加密
 *
 * 场景：应用可以一次性加载所有数据到内存中，直接对所有数据进行对称加密计算。
 *
 * 功能：单一分组数据的加密操作。
 *
 * 注1：待加密数据被作为字节数据进行编码，内部不对待加密数据做其他编码处理（例如中文编码）
 * 注2：DES/3DES算法不支持CTR和GCM模式
 *
 * @param config    [in] 非空，分组密码算法配置：算法，模式，密钥，向量，填充方式等
 * @param data      [in] 非空，待加密数据
 * @param data_len  [in] 非负数，待加密数据长度
 * @param enc       [out] 可空，二进制密文数据（传NULL时，由enc_len返回缓冲区大小，建议设置为[data_len+ISEC_MAX_KEY_LEN]字节）
 * @param enc_len   [in] 非空，密文数据缓冲区大小 [out] 非空，密文长度
 * @param tag_len   [in] 非负数，若对称算法模式为GCM，则需要指定输出的认证数据的长度，否则设置为0
 * @param tag       [out] 可空，若对称算法模式为GCM，输出认证数据，缓冲区大小建议设置为[tag_len]字节
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_encrypt(const isec_cipher_config_t *config, const void *data, int data_len,
    unsigned char *enc, int *enc_len, int tag_len, unsigned char *tag);

/**
 * @brief 多组数据加密初始化
 *
 * 场景：若应用需要对所有数据进行对称加密计算，但每次只能加载部分数据到内存。
 *
 * 功能：多组数据加密初始化。设置数据加密的算法相关参数。
 *
 * 注1：DES/3DES算法不支持CTR和GCM模式
 *
 * @param config  [in] 非空，分组密码算法配置：算法，模式，密钥，向量，填充方式等
 * @param cipher  [out] 非空，分组密码句柄
 * @return int    [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_encrypt_init(const isec_cipher_config_t *config, isec_cipher_t **cipher);

/**
 * @brief 多组数据加密
 *
 * 场景：若应用需要对所有数据进行对称加密计算，但无法一次性加载所有数据到内存，需要实时
 *      加载流数据到内存并加密输出。
 *
 * 功能：多个分组数据的加密操作。
 *
 * 流程：
 * 1.首次调用本接口之前，必须调用isec_crypto_encrypt_init初始化加密计算操作；
 * 2.若应用数据加载未结束，应用将本次加载的任意长度数据作为输入，调用本接口进行计算，
 *   内部自动将上次调用缓存的原文和本次输入的原文进行拼接并按照算法分组加密输出，不满足
 *   算法分组的原文部分缓存到句柄，等待下一次数据加载并输入；
 * 3.若应用数据加载已结束，调用isec_crypto_encrypt_final对缓冲区内剩余的原文进行处理
 *   并加密输出；
 *
 * 注1：当输入的数据长度不足该算法的分组长度，则输入数据会暂时缓存在句柄中，并返回密文长度为0。
 * 缓存数据会拼接下一次输入进行加密输出或由isec_crypto_encrypt_final接口填充后加密输出。
 *
 * @param cipher    [in] 非空，分组密码句柄
 * @param data      [in] 非空，待加密数据
 * @param data_len  [in] 非负数，待加密数据长度
 * @param enc       [out] 可空，二进制密文数据（传NULL时，由enc_len返回缓冲区大小，建议设置为[data_len+ISEC_MAX_KEY_LEN]字节）
 * @param enc_len   [in] 非空，密文数据缓冲区大小 [out] 非空，密文长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_encrypt_update(isec_cipher_t *cipher,
    const void *data, int data_len, unsigned char *enc, int *enc_len);

/**
 * @brief 多组数据加密结束
 *
 * 场景：若应用需要对所有数据进行对称加密计算，但每次只能加载部分数据到内存。
 *
 * 功能：结束多个分组数据的加密，返回剩余加密结果。
 *
 * 流程：完成后，调用isec_crypto_encrypt_clean关闭分组密码句柄，释放内存。
 *
 * @param cipher    [in] 非空，分组密码句柄
 * @param enc       [out] 可空，二进制密文数据（传NULL时，由enc_len返回缓冲区大小，建议设置为[ISEC_MAX_KEY_LEN]字节）
 * @param enc_len   [in] 非空，缓冲区大小 [out] 非空，密文长度
 * @param tag_len   [in] 可空，若对称算法模式为GCM，则需要指定输出的认证数据的长度，一般在0-16字节
 * @param tag       [out] 可空，若对称算法模式为GCM，输出认证数据，缓冲区大小建议设置为[tag_len]字节
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_encrypt_final(isec_cipher_t *cipher, unsigned char *enc, int *enc_len,
    int tag_len, unsigned char *tag);

/**
 * @brief 关闭多组数据加密句柄
 *
 * 功能：关闭isec_crypto_encrypt_init创建的句柄，释放内存。
 *
 * @param cipher  [in] 非空，分组密码句柄
 * @return int    [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_encrypt_clean(isec_cipher_t *cipher);

/**
 * @brief 单组数据解密
 *
 * 场景：应用可以一次性加载所有数据到内存中，直接对所有数据进行对称解密计算
 *
 * 功能：单一分组数据的解密操作。
 *
 * 注1：待解密数据被作为字节数据进行编码，内部不对待解密数据做其他编码处理（例如中文编码）
 * 注2：DES/3DES算法不支持CTR和GCM模式
 *
 * @param config    [in] 非空，分组密码算法配置：算法，模式，密钥，向量，填充方式等
 * @param enc       [in] 非空，待解密密文
 * @param enc_len   [in] 非负数，待解密密文长度
 * @param tag       [in] 可空，若对称算法模式为GCM，则需要输入认证数据
 * @param tag_len   [in] 非负数，若对称算法模式为GCM，则需要输入认证数据长度
 * @param dec       [out] 可空，明文数据（传NULL时，由dec_len返回缓冲区大小，建议设置为[enc_len]字节）
 * @param dec_len   [in] 非空，明文数据缓冲区大小 [out] 非空，明文长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_decrypt(const isec_cipher_config_t *config,
    const unsigned char *enc, int enc_len, const unsigned char *tag, int tag_len,
    unsigned char *dec, int *dec_len);

/**
 * @brief 多组数据解密初始化
 *
 * 场景：若应用需要对所有密文进行对称解密计算，但每次只能加载部分密文到内存。
 *
 * 功能：数据解密初始化。设置数据解密的算法相关参数。
 *
 * 注1：DES/3DES算法不支持CTR和GCM模式
 *
 * @param config  [in] 非空，分组密码算法配置：算法，模式，密钥，向量，填充方式等
 * @param cipher  [out] 非空，分组密码句柄
 * @return int    [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_decrypt_init(const isec_cipher_config_t *config, isec_cipher_t **cipher);

/**
 * @brief 多组数据解密
 *
 * 场景：若应用需要对所有密文进行对称解密计算，但每次只能加载部分密文到内存。
 *
 * 功能：多个分组数据的解密操作。
 *
 * 流程：
 * 1.首次调用本接口之前，必须调用isec_crypto_deccrypt_init初始化解密计算操作；
 * 2.若应用密文加载未结束，应用将本次加载的任意长度密文作为输入，调用本接口进行计算，
 * 内部自动将缓存的密文和输入的密文进行拼接并按照算法分组解密输出，不满足算法分组的
 * 密文部分缓存到句柄，等待下一次密文加载并输入；
 * 3.若应用密文加载已结束，调用isec_crypto_decrypt_final对缓冲区内剩余的密文进行处理
 * 并解密输出；
 *
 * @param cipher    [in] 非空，分组密码句柄
 * @param enc       [in] 非空，待解密密文
 * @param enc_len   [in] 非负数，待解密密文长度
 * @param dec       [out] 可空，明文数据（传NULL时，由dec_len返回缓冲区大小，建议设置为[ISEC_MAX_KEY_LEN]字节）
 * @param dec_len   [in] 非空，明文数据缓冲区大小 [out] 非空，明文长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_decrypt_update(isec_cipher_t *cipher, const void *enc, int enc_len,
    unsigned char *dec, int *dec_len);

/**
 * @brief 多组数据解密结束
 *
 * 场景：若应用需要对所有密文进行对称解密计算，但每次只能加载部分密文到内存。
 *
 * 功能：结束多个分组数据的解密，返回剩余解密结果。
 *
 * 流程：完成后，调用isec_crypto_decrypt_clean关闭分组密码句柄，释放内存。
 *
 * @param cipher    [in] 非空，分组密码句柄
 * @param tag       [in] 可空，若对称算法模式为GCM，则需要输入认证数据
 * @param tag_len   [in] 非负数，若对称算法模式为GCM，则需要输入认证数据长度
 * @param dec       [in] 可空，明文数据（传NULL时，由dec_len返回缓冲区大小，建议设置为[ISEC_MAX_KEY_LEN]字节）
 * @param dec_len   [in] 非空，明文数据缓冲区大小 [out] 非空，明文长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_decrypt_final(isec_cipher_t *cipher,
    const unsigned char *tag, int tag_len, void *dec, int *dec_len);

/**
 * @brief 关闭多组数据解密句柄
 *
 * 功能：关闭isec_crypto_encrypt_init创建的句柄，释放内存。
 *
 * @param cipher  [in] 非空，分组密码句柄
 * @return int    [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_decrypt_clean(isec_cipher_t *cipher);

/**
 * @brief 公钥加密
 *
 * 功能：使用外部传入公钥对输入数据做加密运算并输出结果。
 *
 * 注1：使用RSA加密，不填充时：数据长度不能超过密钥长度，PKCS1填充时：数据长度不能超过密钥长度-11字节
 *
 * @param config    [in] 非空，非对称加密配置
 * @param data      [in] 非空，待加密数据
 * @param data_len  [in] 非负数，待加密数据长度
 * @param enc       [out] 可空，密文数据缓冲区（传NULL时，由enc_len返回缓冲区大小）
 * @param enc_len   [in] 非空，密文数据缓冲区大小 [out] 非空，密文长度

 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_public_encrypt(const isec_public_encrypt_config_t *config,
    const void *data, int data_len, unsigned char *enc, int *enc_len);

/**
 * @brief 私钥解密
 *
 * 功能：使用外部传入私钥对输入数据做解密运算并输出结果。
 *
 * @param config    [in] 非空，非对称加密配置
 * @param enc       [in] 非空，待解密数据
 * @param enc_len   [in] 非负数，待解密数据长度
 * @param dec       [out] 可空，明文数据缓冲区（传NULL时，由dec_len返回缓冲区大小，建议设置为[enc_len]字节）
 * @param dec_len   [in] 非空，明文数据缓冲区大小 [out] 非空，明文数据长度

 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_private_decrypt(const isec_public_encrypt_config_t *config,
    const unsigned char *enc, int enc_len, void *dec, int *dec_len);

/**
 * @brief 私钥签名
 *
 * 功能：使用外部传入私钥，对传入的原文或摘要数据进行签名。
 *
 * 注1：
 *   若未配置no_hash时，表示待签名数据data为原文，内部对原文数据计算摘要后，对摘要进行签名
 *   若已配置no_hash时，表示待签名数据data为摘要，直接对待签名数据进行签名
 * 注2：
 *   若签名密钥类型为SM2且摘要算法为SM3，则数据摘要方式为SM2签名预处理，预处理过程遵循GM/T 0009。
 *
 * @param config    [in] 非空，签名配置
 * @param data      [in] 非空，待签名数据
 * @param data_len  [in] 非负数，待签名数据长度
 * @param sign      [out] 可空，二进制签名值缓冲区（传NULL时，由sign_len返回缓冲区大小，建议设置为[ISEC_MAX_KEYPAIR_LEN]字节）
 * @param sign_len  [in] 非空，签名值缓冲区大小 [out] 非空，签名值长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_private_sign(const isec_pkcs1_sign_config_t *config,
    const void *data, int data_len, unsigned char *sign, int *sign_len);

/**
 * @brief 公钥验签
 *
 * 功能：使用外部传入公钥或证书，对传入的原文或摘要数据进行验签名。
 *
 * 注1：
 *   若未配置no_hash时，表示待签名数据data为原文，内部对原文数据计算摘要后，对摘要进行签名
 *   若已配置no_hash时，表示待签名数据data为摘要，直接对待签名数据进行签名
 * 注2：
 *   若签名密钥类型为SM2且摘要算法为SM3，则数据摘要方式为SM2签名预处理，预处理过程遵循GM/T 0009。
 *
 * @param config    [in] 非空，签名配置
 * @param data      [in] 非空，待验签数据
 * @param data_len  [in] 非负数，待验签数据长度
 * @param sign      [in] 非空，二进制签名值
 * @param sign_len  [in] 非空，签名值长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_public_verify(const isec_pkcs1_sign_config_t *config,
    const void *data, int data_len, const unsigned char *sign, int sign_len);

/**
 * @brief 密钥协商
 *
 * 功能：根据本端的加密公私钥、临时公私钥和对端的加密公钥、临时公钥，协商出指定长度的密钥，
 * 双方协商的密钥一致。
 *
 * @param config   [in] 密钥协商配置
 * @param key_len  [in] 非负数，指定输出的密钥长度
 * @param key      [out] 非空，密钥缓冲区，缓冲区大小建议设置为[key_len]字节
 * @return int     [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_key_exchange(const isec_key_exchange_config_t *config,
    int key_len, unsigned char *key);

/**
 * @brief 数字信封加密
 *
 * 功能：使用外部传入的加密证书对输入数据加密，并输出数字信封。
 *
 * 注1：SM2仅支持SM4算法，RSA支持除SM4外的所有算法
 * 注2：所有算法均不支持CTR和GCM模式
 * 注3：DES/3DES算法仅支持CBC模式
 * 注4：对接信安签名服务器时，若使用国密算法其无法正确解密，尝试将数字信封配置中SM4算法的模式需要设置为none，实际效果为SM4-CBC模式。
 *
 * @param config    [in] 非空，数字信封配置
 * @param data      [in] 非空，待加密数据
 * @param data_len  [in] 非负数，待加密数据长度
 * @param enc       [out] 可空，DER编码二进制数据的数字信封（传NULL时，由enc_len返回缓冲区大小）
 * @param enc_len   [in] 非空，数字信封缓冲区大小 [out] 非空，数字信封长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_pkcs7_encrypt(const isec_pkcs7_encrypt_config_t *config,
    const void *data, int data_len, unsigned char *enc, int *enc_len);
/**
 * @brief 数字信封解密
 *
 * 功能：使用外部传入私钥对做数字信封解密并输出原文。
 *
 * @param config    [in] 非空，数字信封配置
 * @param enc       [in] 非空，数字信封数据，同时支持DER编码二进制数据和Base64编码字符串
 * @param enc_len   [in] 非负数，数字信封长度
 * @param dec       [out] 可空，明文长度（传NULL时，由dec_len返回缓冲区大小，建议设置为[enc_len]字节）
 * @param dec_len   [in] 非空，明文缓冲区大小 [out] 非空，明文长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_pkcs7_decrypt(const isec_pkcs7_encrypt_config_t *config,
    const void *enc, int enc_len, void *dec, int *dec_len);

/**
 * @brief PKCS#7签名
 *
 * 功能：使用外部传入私钥对原文数据进行签名，输出P7签名。
 *
 * 注：
 *   若未配置with_data时，使用detach签名，输出的签名值中不包含原文，缓冲区大小可以不考虑原文长度。
 *   若已配置with_data时，使用attach签名，输出的签名值中包含原文。
 *
 * @param config    [in] 非空，PKCS#7签名配置：私钥，摘要算法等
 * @param data      [in] 非空，待签名原文
 * @param data_len  [in] 非负数，待签名原文长度
 * @param sign      [in] 可空，DER编码的二进制签名值数据（传NULL时，由sign_len返回缓冲区大小）
 * @param sign_len  [in] 非空，签名值缓冲区大小 [out] 签名值长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_pkcs7_sign(const isec_pkcs7_sign_config_t *config,
    const void *data, int data_len, unsigned char *sign, int *sign_len);

/**
 * @brief PKCS#7 验签
 *
 * 功能：使用外部传入证书验证P7签名。
 *
 * 注1：
 *   若with_data非0，则以P7内部原文作为待验签原文，忽略传入的data和data_len（detach签名内部不包含原文，该情况一定验签失败）
 *   若with_data为0，则以接口输入数据作为待验签原文（attach签名内部包含原文被忽略）
 *
 * @param config    [in] PKCS#7签名配置：私钥等
 * @param data      [in] 可空，待验签原文
 * @param data_len  [in] 非负数，待验签原文长度
 * @param sign      [in] 非空，签名值数据，同时支持DER编码二进制数据和Base64编码字符串
 * @param sign_len  [in] 非空，签名值长度
 * @return int      [out] 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_crypto_pkcs7_verify(const isec_pkcs7_sign_config_t *config,
    const void *data, int data_len, const void *sign, int sign_len);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __ISEC_CRYPTO_H */

