#ifndef __ISEC_DEF_H
#define __ISEC_DEF_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <stdlib.h>

#ifndef ISEC_API
# define ISEC_API
#endif /* ISEC_API */

/* ISEC 错误码 */
/* 接口 */
#define isec_err_none                   0x00000000      /* 成功 */
#define isec_err_fail                   0x0A000001      /* 失败 */
#define isec_err_internal               0x0A000002      /* 内部错误 */
#define isec_err_param_null             0x0A000003      /* 参数为空 */
#define isec_err_param_invalid          0x0A000004      /* 参数无效 */
#define isec_err_buffer_too_small       0x0A000005      /* 缓冲区太小 */
#define isec_err_not_support            0x0A000006      /* 当前版本不支持 */
/* 系统 */
#define isec_err_memory                 0x0A010001      /* 内存错误 */
#define isec_err_file_not_exist         0x0A010002      /* 文件不存在 */
/* 算法 */
#define isec_err_key_invalid            0x0A020001      /* 无效的密钥 */
#define isec_err_iv_invalid             0x0A020002      /* 无效的初始化向量 */
#define isec_err_aad_invalid            0x0A020003      /* 无效的附加认证数据 */
#define isec_err_tag_invalid            0x0A020004      /* 无效的TAG */
#define isec_err_encode_fail            0x0A020005      /* 编码失败 */
#define isec_err_decode_fail            0x0A020006      /* 解码失败 */
#define isec_err_hash_fail              0x0A020007      /* 摘要失败 */
#define isec_err_hmac_fail              0x0A020008      /* HMAC失败 */
#define isec_err_cmac_fail              0x0A020009      /* CMAC失败 */
#define isec_err_sign_fail              0x0A02000A      /* 签名失败 */
#define isec_err_verify_fail            0x0A02000B      /* 验证签名失败 */
#define isec_err_encrypt_fail           0x0A02000C      /* 加密失败 */
#define isec_err_decrypt_fail           0x0A02000D      /* 解密失败 */
/* 密钥对 */
#define isec_err_keypair_parse_fail     0x0A030001      /* 密钥对数据解析失败 */
#define isec_err_keypair_not_match      0x0A030002      /* 密钥对不匹配 */
/* 证书 */
#define isec_err_cert_parse_fail        0x0A040001      /* 证书数据解析失败 */
#define isec_err_cert_not_exist         0x0A040002      /* 证书不存在 */
#define isec_err_cert_not_yet_valid     0x0A040003      /* 证书未生效 */
#define isec_err_cert_expired           0x0A040004      /* 证书过期 */
#define isec_err_cert_revoke            0x0A040005      /* 证书作废 */
#define isec_err_cert_verify_fail       0x0A040006      /* 证书验证失败 */
/* 通信 */
#define isec_err_net_connect_fail       0x0A070001      /* 连接失败 */
#define isec_err_net_disconnect         0x0A070002      /* 连接断开 */
#define isec_err_net_send_again         0x0A070003      /* 套接字写重试，应用需要监听套接字的写事件，事件触发后再次调用当前接口 */
#define isec_err_net_recv_again         0x0A070004      /* 套接字读重试，应用需要监听套接字的读事件，事件触发后再次调用当前接口 */

/**
 * @brief 常量宏定义
 *
 */
#define ISEC_MAX_MD_LEN                     64                          /* 最大摘要长度 */
#define ISEC_MAX_KEY_LEN                    64                          /* 最大密钥长度 */
#define ISEC_MAX_IV_LEN                     64                          /* 最大向量长度 */
#define ISEC_MAX_KEYPAIR_LEN                4096                        /* 最大非对称密钥长度 */
#define ISEC_MAX_CERTIFICATE_LEN            8192                        /* 最大证书长度 */
#define ISEC_MAX_BASE64_ENCODE_LEN(len)     (((len)+2)/3*4+1)           /* 最大Base64编码后长度（入参：待编码数据长度） */
#define ISEC_MAX_BASE64_DECODE_LEN(len)     (len)                       /* 最大Base64编码后长度（入参：待编码数据长度） */
#define ISEC_MAX_ENCRYPT_LEN(len)           ((len)+ISEC_MAX_KEY_LEN)    /* 最大对称加密后长度（入参：待加密数据长度） */
#define ISEC_MAX_DECRYPT_LEN(len)           (len)                       /* 最大对称解密后长度（入参：待解密数据长度） */

/**
 * @brief 摘要算法句柄
 *
 */
typedef struct isec_hash_s isec_hash_t;

/**
 * @brief 摘要算法
 *
 * 定义了摘要算法的类型。
 */
typedef enum {
    isec_hash_alg_none   = 0,   /* 摘要算法未定义 */
    isec_hash_alg_sm3    = 1,   /* 摘要算法SM3 */
    isec_hash_alg_md5    = 2,   /* 摘要算法MD5 */
    isec_hash_alg_sha1   = 3,   /* 摘要算法SHA1 */
    isec_hash_alg_sha256 = 4,   /* 摘要算法SHA256 */
    isec_hash_alg_sha384 = 5,   /* 摘要算法SHA384 */
    isec_hash_alg_sha512 = 6    /* 摘要算法SHA512 */
} isec_hash_alg_e;

/**
 * @brief HMAC消息鉴别码句柄
 *
 */
typedef struct isec_hmac_s isec_hmac_t;

/**
 * @brief CMAC消息鉴别码句柄
 *
 */
typedef struct isec_cmac_s isec_cmac_t;

/**
 * @brief 对称算法句柄
 *
 */
typedef struct isec_cipher_s isec_cipher_t;

/**
 * @brief 对称算法
 *
 */
typedef enum {
    isec_cipher_alg_none     = 0,   /* 对称算法未定义 */
    isec_cipher_alg_sm4      = 1,   /* SM4，密钥长度为128bits */
    isec_cipher_alg_des      = 2,   /* DES，密钥长度为64bits */
    isec_cipher_alg_3des_112 = 3,   /* 3DES，密钥长度为128bits */
    isec_cipher_alg_3des_168 = 4,   /* 3DES，密钥长度为192bits */
    isec_cipher_alg_aes_128  = 5,   /* AES-128，密钥长度为128bits */
    isec_cipher_alg_aes_192  = 6,   /* AES-192，密钥长度为192bits */
    isec_cipher_alg_aes_256  = 7    /* AES-256，密钥长度为256bits */
} isec_cipher_alg_e;

/**
 * @brief 对称算法模式
 *
 */
typedef enum {
    isec_cipher_mode_none   = 0,  /* 模式未定义 */
    isec_cipher_mode_ecb    = 1,  /* 加密模式ECB */
    isec_cipher_mode_cbc    = 2,  /* 加密模式CBC */
    isec_cipher_mode_cfb    = 3,  /* 加密模式CFB */
    isec_cipher_mode_ofb    = 4,  /* 加密模式OFB */
    isec_cipher_mode_ctr    = 5,  /* 加密模式CTR */
    isec_cipher_mode_gcm    = 6   /* 加密模式GCM */
} isec_cipher_mode_e;

/**
 * @brief 对称算法填充
 *
 */
typedef enum isec_cipher_padding_e {
    isec_cipher_padding_none  = 0,  /* 不填充 */
    isec_cipher_padding_pkcs7 = 1   /* PKCS7填充（兼容PKCS5填充） */
} isec_cipher_padding_e;

/**
 * @brief 非对称密钥对类型
 *
 * 定义了非对称算法的密钥对类型。
 */
typedef enum {
    isec_keypair_type_none   = 0, /* 密钥类型未定义 */
    isec_keypair_type_sm2    = 1, /* SM2，密钥强度支持 256 bits */
    isec_keypair_type_rsa    = 3  /* RSA，密钥强度支持 1024/2048/4096 bits */
} isec_keypair_type_e;

/**
 * @brief 密钥强度
 *
 * 定义了非对称算法的密钥对强度。
 */
typedef enum {
    isec_keypair_strength_none = 0, /* 密钥强度未定义 */
    isec_keypair_strength_256  = 1, /* 密钥强度为长度  256 bits */
    isec_keypair_strength_1024 = 2, /* 密钥强度为长度 1024 bits */
    isec_keypair_strength_2048 = 3, /* 密钥强度为长度 2048 bits */
    isec_keypair_strength_4096 = 4  /* 密钥强度为长度 4096 bits */
} isec_keypair_strength_e;

/**
 * @brief 数据格式
 *
 * 定义了数据的格式。
 */
typedef enum {
    isec_format_none   = 0, /* 数据格式未定义 */
    isec_format_bin    = 1, /* 字节数据，二进制字节数据 */
    isec_format_der    = 3, /* 字节数据，对二进制字节数据进行ASN.1(der)编码后的二进制字节数据 */
    isec_format_base64 = 4, /* 字符串数据，对DER数据进行Base64编码，单行输出的字符串数据 */
    isec_format_pem    = 5  /* 字符串数据，对DER数据进行Base64编码，带BEGIN/END标签的多行输出的字符串数据 */
} isec_format_e;

/**
 * @brief 非对称密钥对句柄
 *
 */
typedef void isec_keypair_t;

/**
 * @brief RSA加密填充方式
 *
 */
typedef enum isec_rsa_padding_e {
    isec_rsa_padding_none  = 0,  /* 不填充 */
    isec_rsa_padding_pkcs1 = 1   /* PKCS1填充 */
} isec_rsa_padding_e;

/**
 * @brief 证书句柄
 *
 */
typedef void isec_certificate_t;

/**
 * @brief 证书信息
 *
 * 定义了证书信息。
 */
typedef enum {
    isec_certificate_info_none = 0,
    isec_certificate_info_serial,            /* 证书序列号 */
    isec_certificate_info_issuer,            /* 证书颁发者 */
    isec_certificate_info_validity_before,   /* 证书有效期开始时间，时间格式为RFC3339(例如：2021-11-25T10:17:11+0000) */
    isec_certificate_info_validity_after,    /* 证书有效期结束时间，时间格式为RFC3339(例如：2026-11-25T11:17:11+0000)  */
    isec_certificate_info_subject,           /* 证书使用者 */
    isec_certificate_info_pubkey,            /* 证书公钥，该数据（SM2是裸公钥，RSA是DER编码公钥）经过Base64编码输出 */
    isec_certificate_info_tbs,               /* 待签数据，该数据经过Base64编码输出 */
    isec_certificate_info_signature,         /* 签名值，该数据经过Base64编码输出 */
    isec_certificate_info_signature_alg      /* 签名算法（例如："sm2sign-with-sm3"）（注：算法名可能变化，当前仅保证出现rsa/sha256/sm2/sm3等不区分大小写的关键词） */
} isec_certificate_info_e;

/**
 * @brief 证书请求配置
 *
 * 使用指定的密钥对keypair（必须包含私钥）和摘要算法，产生主题为dn的证书请求。
 */
typedef struct isec_csr_config_s {
    /* 必选参数 */
    isec_keypair_t  *keypair;   /* 私钥，由isec_keypair_unmarshal()接口将数据转换为密钥对句柄 */
    isec_hash_alg_e  hash;      /* 摘要算法 */
    /* 可选 */
    const char      *dn;        /* 证书请求的主题 */
} isec_csr_config_t;

/**
 * @brief 摘要算法配置
 *
 # 若对数据进行SM2签名预处理操作，则摘要算法为SM3，且必须额外传入SM2类型的密钥对（包含公钥即可）。
 */
typedef struct isec_hash_config_s {
    /* 必选 */
    isec_hash_alg_e  hash;        /* 摘要算法 */
    /* 可选 */
    isec_keypair_t  *sm2;         /* 若摘要算法为SM3且配置SM2密钥对，则计算摘要时会对数据进行国密预处理操作。 */
    const void      *id;          /* 若摘要算法为SM3且配置SM2密钥对，则计算摘要时对数据进行国密预处理操作。为空默认为"1234567812345678" */
    int              id_len;      /* 预处理身份标识长度 */
} isec_hash_config_t;

/**
 * @brief 对称算法配置
 *
 * 注1：当前版本des/3des算法不支持ctr/gcm模式
 */
typedef struct isec_cipher_config_s {
    /* 必选 */
    isec_cipher_alg_e       alg;        /* 对称算法 */
    isec_cipher_mode_e      mode;       /* 对称算法模式 */
    /* 可选 */
    isec_cipher_padding_e   padding;    /* 对称算法填充 */
    const void             *key;        /* 对称密钥 */
    int                     key_len;    /* 对称密钥长度 */
    const void             *iv;         /* 初始化向量。若对称算法模式非ECB，则必须传入该参数 */
    int                     iv_len;     /* 初始化向量长度 */
    const void             *aad;        /* 仅GCM算法需要配置，一般在0-16字节 */
    int                     aad_len;    /* 仅GCM算法需要配置 */
} isec_cipher_config_t;

/**
 * @brief HMAC消息鉴别码配置
 *
 */
typedef struct isec_hmac_config_s {
    /* 必选 */
    isec_hash_alg_e     hash;       /* 摘要算法 */
    const void         *key;        /* 认证密钥 */
    int                 key_len;    /* 认证密钥长度，密钥的长度不受限 */
} isec_hmac_config_t;

/**
 * @brief CMAC消息鉴别码配置
 *
 */
typedef struct isec_cmac_config_s {
    /* 必选 */
    isec_cipher_alg_e     cipher;   /* 对称算法 */
    const void           *key;      /* 认证密钥 */
    int                   key_len;  /* 认证密钥长度，密钥的长度于对称算法相关 */
} isec_cmac_config_t;

/**
 * @brief 公钥加密配置
 *
 */
typedef struct isec_public_encrypt_config_s {
    /* 必选 */
    isec_keypair_t     *keypair;      /* 非对称密钥对 */
    /* 可选 */
    isec_rsa_padding_e  rsa_padding;  /* 若密钥对类型为RSA，用于控制输入数据的填充方式；其他密钥类型则忽略 */
    int                 no_asn1;      /* 若密钥对类型为SM2，用于控制密文输出格式；其他密钥类型则忽略 */
                                      /* 0 - SM2密文经过ASN.1(der)编码  */
                                      /* 1 - SM2密文不经过ASN.1(der)编码 */
    int                 no_c1c3c2;    /* 若密钥对类型为SM2且no_asn1设置为1，用于控制密文输出格式；其他密钥类型则忽略 */
                                      /* 0 - SM2密文格式为C1 C3 C2 */
                                      /* 1 - SM2密文格式为C1 C2 C3 */
} isec_public_encrypt_config_t;

/**
 * @brief 私钥签名配置
 *
 */
typedef struct isec_pkcs1_sign_config_s {
    /* 必选 */
    isec_keypair_t  *keypair;   /* 非对称密钥对 */
    isec_hash_alg_e  hash;      /* 签名使用的摘要算法 */
    /* 可选 */
    int              no_hash;   /* 用于控制是否对输入数据进行摘要计算 */
                                /* 0 - 将输入数据作为原文，内部计算摘要后，再进行签名 */
                                /* 1 - 将输入数据作为摘要，直接进行签名 */
    int              no_asn1;   /* 若密钥对类型为SM2，用于控制签名输出格式；其他密钥类型则忽略 */
                                /* 0 - 若密钥对类型为SM2且签名值类型为裸签名，则签名经过ASN.1(der)编码  */
                                /* 1 - 若密钥对类型为SM2且签名值类型为裸签名，裸签名不经过ASN.1(der)编码 */
} isec_pkcs1_sign_config_t;

/**
 * @brief 密钥协商配置
 *
 */
typedef struct isec_key_exchange_config_s {
    /* 必选 */
    int              self_is_initiator;  /* 本端是否为发起方（0 - 接收方，1 - 发起方） */
    const void      *self_id;            /* 本端ID */
    int              self_id_len;
    isec_keypair_t  *self_enc_prikey;    /* 本端加密私钥 */
    isec_keypair_t  *self_tmp_prikey;    /* 本端临时私钥 */
    const void      *peer_id;            /* 对端ID */
    int              peer_id_len;
    isec_keypair_t  *peer_enc_pubkey;    /* 对端加密公钥 */
    isec_keypair_t  *peer_tmp_pubkey;    /* 对端临时公钥 */
} isec_key_exchange_config_t;

/**
 * @brief PKCS#7 数字信封签名格式
 *
 */
typedef enum isec_pkcs7_encrypt_sign_format_e {
    isec_pkcs7_encrypt_sign_format_none    = 0,  /* 数字信封不做签名 */
    isec_pkcs7_encrypt_sign_format_raw     = 1,  /* PKCS#7标准带签名数字信封格式，信封签名字段数据：签名(摘要(原文))，即私钥裸签名 */
    isec_pkcs7_encrypt_sign_format_netsafe = 2,  /* PKCS#7标准带签名数字信封格式，信封签名字段数据：签名(摘要(对称加密(原文)))，对接NetSafe产品 */
    isec_pkcs7_encrypt_sign_format_csp     = 3,  /* 微软CSP带签名数字信封格式，对原文先attached签名，再对attach签名进行数字信封加密 */
} isec_pkcs7_encrypt_sign_format_e;

/**
 * @brief PKCS#7数字信封配置
 *
 */
typedef struct isec_pkcs7_encrypt_config_s {
    /* 可选，加密相关配置 */
    isec_cipher_alg_e                   alg;                /* 对称算法类型，加密时必须传入，解密时忽略 */
    isec_cipher_mode_e                  mode;               /* 对称算法模式，加密时必须传入，解密时忽略 */
    isec_keypair_t                     *keypair;            /* 接收者非对称私钥，加密时忽略，解密时必须传入 */
    isec_certificate_t                 *certificate;        /* 接收者加密证书，加密时必须传入，解密时忽略 */
    /* 可选，签名相关配置 */
    isec_pkcs7_encrypt_sign_format_e    with_sign;          /* 指定数字信封的签名类型 */
    isec_hash_alg_e                     sign_hash;          /* 签名摘要算法，若with_sign非0，加密时必须传入，解密时忽略 */
    isec_keypair_t                     *sign_keypair;       /* 签名者非对称私钥，若with_sign非0，加密时必须传入，解密时忽略 */
    isec_certificate_t                 *sign_certificate;   /* 签名者加密证书，若with_sign非0，加密时必须传入，解密时忽略 */
} isec_pkcs7_encrypt_config_t;

/**
 * @brief PKCS#7签名配置
 *
 */
typedef struct isec_pkcs7_sign_config_s {
    /* 可选 */
    isec_certificate_t  *certificate; /* 签名者证书，仅签名时必须传入 */
    isec_keypair_t      *keypair;     /* 签名者私钥，仅签名时必须传入 */
    isec_hash_alg_e      hash;        /* 摘要算法，仅签名时必须传入 */
    int                  with_data;   /* 用于控制签名/验签原文数据 */
                                      /* 0 - 签名时输出attach签名，验签时以外部输入原文作为待验签原文 */
                                      /* 1 - 签名时输出detach签名，验签时以签名内部原文作为待验签原文 */
} isec_pkcs7_sign_config_t;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __ISEC_DEF_H */

