#ifndef __ISEC_KE_H
#define __ISEC_KE_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "isec.h"

/**
 * @brief 密钥协商句柄
 *
 */
typedef struct isec_ke_s isec_ke_t;

/**
 * @brief 创建密钥协商句柄
 *
 * 应用的一次会话需要创建一次密钥协商句柄，密钥协商流程完成后，该会话的应用数据使用该句柄进行
 * 加密、解密等操作，会话结束后即可销毁密钥协商句柄。
 *
 * @param ke  [out] 非空，密钥协商句柄
 *
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_new(isec_ke_t **ke);

/**
 * @brief 销毁密钥协商句柄
 *
 * @param ke  [in] 非空，密钥协商句柄
 *
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_free(isec_ke_t *ke);

/**
 * @brief 备份密钥协商句柄
 *
 * 导出当前密钥协商句柄内部数据，其数据仅用于C接口的isec_ke_unmarshal恢复句柄，适用于密钥句柄无法在应用系统直接传递的场景。
 *
 * 注1：数据中可能包含私钥、对称密钥等敏感信息，该数据仅限本地使用，禁止传输到其他终端
 *
 * @param ke        [in] 密钥协商句柄
 * @param data      [out] 可空，密钥协商数据字符串（传NULL时，由data_len返回缓冲区大小）
 * @param data_len  [in] 非空，缓冲区大小 [out] 非空，密钥协商数据长度
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_marshal(isec_ke_t *ke, char *data, int *data_len);

/**
 * @brief 恢复密钥协商句柄
 *
 * 将isec_ke_marshal接口备份出的数据恢复为密钥协商句柄
 *
 * @param data      [in] 非空，密钥协商数据字符串
 * @param data_len  [in] 正数，密钥协商数据长度
 * @param ke        [out] 密钥协商句柄
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_unmarshal(const char *data, int data_len, isec_ke_t **ke);

/**
 * @brief 加载私钥
 *
 * 服务端必选：
 *   加载私钥，用于密钥协商流程中对数据的加密、解密、签名、验签。
 *
 * @param ke           [in] 非空，密钥协商句柄
 * @param sig_key      [in] 可空，签名私钥数据
 * @param sig_key_len  [in] 可空，签名私钥长度
 * @param enc_key      [in] 可空，加密私钥数据
 * @param enc_key_len  [in] 可空，加密私钥长度
 * @param password     [in] 可空，私钥口令
 *
 * @return int     成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_set_private_key(isec_ke_t *ke, const void *sig_key, int sig_key_len,
    const void *enc_key, int enc_key_len, const char *password);

/**
 * @brief 加载公钥
 *
 * 客户端必选：
 *   若服务端无证书，则客户端预置服务端公钥，用于密钥协商流程中的加密、验签。
 *
 * @param ke           [in] 非空，密钥协商句柄
 * @param sig_key      [in] 可空，签名公钥数据，数据格式参考isec_keypair_marshal()说明
 * @param sig_key_len  [in] 可空，签名公钥长度
 * @param enc_key      [in] 可空，加密公钥数据，数据格式参考isec_keypair_marshal()说明
 * @param enc_key_len  [in] 可空，加密公钥长度
 * @param password     [in] 可空，公钥口令
 *
 * @return int     		  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_set_public_key(isec_ke_t *ke, const void *sig_key, int sig_key_len,
    const void *enc_key, int enc_key_len, const char *password);

/**
 * @brief 加载证书
 *
 * 服务端可选：
 *   在服务端响应密钥协商(isec_ke_server_hello)阶段，以DER编码的X509证书的格式导出，并且作为签名原文参与签名
 *
 * @param ke            [in] 非空，密钥协商句柄
 * @param sig_cert      [in] 非空，签名证书数据，数据格式参考isec_certificate_unmarshal()说明
 * @param sig_cert_len  [in] 非空，签名证书长度
 * @param enc_cert      [in] 非空，加密证书数据，数据格式参考isec_certificate_unmarshal()说明
 * @param enc_cert_len  [in] 非空，加密证书长度
 * @param password      [in] 可空，证书口令
 *
 * @return int     		成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_set_certificate(isec_ke_t *ke, const void *sig_cert, int sig_cert_len,
    const void *enc_cert, int enc_cert_len, const char *password);

/**
 * @brief 加载可信根证书
 *
 * 客户端可选：
 *   用于客户端密钥协商(isec_ke_client_key_exchange)阶段，验证服务端证书有效性。
 *
 * @param ke       [in] 非空，密钥协商句柄
 * @param ca_file  [in] 非空，根证书文件路径字符串数据，该文件会被加载为根证书
 * @param ca_path  [in] 非空，根证书文件夹路径字符串数据，该文件夹路径的证书文件会被加载为根证书（该目录下需事先执行c_rehash命令）
 *
 * @return int     成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_set_root_certificate(isec_ke_t *ke, const char *ca_file, const char *ca_path);

/**
 * @brief 客户端发起密钥协商
 *
 * 客户端必选：
 *   客户端发起密钥协商，随机产生32字节的随机数字节数据明文。
 * 流程：
 *   调用当前接口产生随机数；
 *   将随机数发送给服务端，由服务端调用isec_ke_server_hello处理客户端随机数；
 * 内部实现：
 *   随机产生32字节的客户端随机数存储在句柄内部，同时导出该数据将其发送给服务端；
 *
 * @param ke             [in] 非空，密钥协商句柄
 * @param clnt_rand      [out] 非空，客户端随机数数据，缓冲区大小建议设置为32字节
 * @param clnt_rand_len  [in] 非空，客户端随机数缓冲区大小 [out] 非空，客户端随机数长度
 *
 * @return int           成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_client_hello(isec_ke_t *ke, unsigned char *clnt_rand, int *clnt_rand_len);

/**
 * @brief 服务端响应密钥协商
 *
 * 服务端必选：
 *   服务端响应密钥协商，保存客户端随机数，产生服务端随机数，并使用私钥对相关数据进行签名；
 * 流程：
 *   从客户端接收32字节客户端数；
 *   调用本接口处理客户端随机数，输出服务端随机数、服务端签名和加密证书（可空）、服务端签名值；
 *   将本接口的参数发送给客户端，客户端调用isec_ke_client_key_exchange进行处理；
 * 内部实现：
 *   保存从客户端接收的32字节客户端随机数到句柄内部；
 *   随机产生32字节的服务端随机数存储在句柄内部，同时导出该数据将其发送给客户端；
 *   若服务端加载签名、加密证书，则输出DER编码的签名证书和加密证书字节数据；若未加载则不输出，证书数据也不参与签名原文；
 *   服务端对【32字节客户端随机数+32字节服务端随机数+DER编码签名证书（可空）+DER编码加密证书（可空）】进行签名并导出签名值；
 *
 * @param ke                 [in] 非空，密钥协商句柄
 * @param clnt_rand          [in] 非空，客户端随机数数据
 * @param clnt_rand_len      [in] 非空，客户端随机数长度
 * @param srvr_rand          [out] 非空，服务端随机数数据，
 * @param srvr_rand_len      [in] 非空，服务端随机数缓冲区大小 [out] 非空，服务端随机数长度
 * @param srvr_sig_cert      [out] 可空，服务端签名证书数据
 * @param srvr_sig_cert_len  [in] 可空，服务端签名证书缓冲区大小 [out] 可空，服务端证书长度
 * @param srvr_enc_cert      [out] 可空，服务端加密证书数据
 * @param srvr_enc_cert_len  [in] 可空，服务端加密证书缓冲区大小 [out] 可空，服务端证书长度
 * @param srvr_sign          [out] 非空，服务端签名值数据
 * @param srvr_sign_len      [in] 非空，服务端签名值缓冲区大小 [out] 非空，服务端签名值长度
 *
 * @return int           成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_server_hello(isec_ke_t *ke, const unsigned char *clnt_rand, int clnt_rand_len,
    unsigned char *srvr_rand, int *srvr_rand_len,
    unsigned char *srvr_sig_cert, int *srvr_sig_cert_len,
    unsigned char *srvr_enc_cert, int *srvr_enc_cert_len,
    unsigned char *srvr_sign, int *srvr_sign_len);

/**
 * @brief 客户端密钥协商
 *
 * 客户端必选：
 *   接收服务端随机数、服务端证书（可空）、服务端签名，调用本接口进行密钥协商并输出预主密钥密文;
 * 流程：
 *   从服务端接收32字节服务端随机数、DER编码的签名和加密证书（可空）、服务端签名
 *   调用本接口产生预主密钥密文
 *   将预主密钥密文发送给服务端，由服务端调用isec_ke_server_key_exchange进行处理并继续后续流程；
 *   客户端密钥协商完成，后续可使用isec_ke_encrypt/isec_ke_decrypt/isec_ke_hmac/isec_ke_cmac 进行算法调用；
 * 内部实现：
 *   若已加载根证书且传入服务端签名证书，则验证服务端签名证书，以验证对端身份，若未加载则不验；
 *   使用服务端签名证书（若空则使用客户端预加载的签名公钥或签名证书或签名私钥）验证服务端签名，以验证数据有效性；
 *   随机产生48字节的预主密钥明文并存储在句柄；
 *   对【32字节客户端随机数+32字节服务端随机数+48字节预主密钥】进行KDF计算对称算法、HMAC、CMAC的密钥；
 *   使用白盒算法加密对称算法、HMAC、CMAC的密钥并保存到句柄中，并从内存中清除明文的密钥；
 *   使用服务端加密证书（若空则使用客户端预加载的加密公钥或加密证书或加密私钥）对48字节预主密钥明文加密并输出。
 *
 * @param ke                  [in] 非空，密钥协商句柄
 * @param srvr_rand           [in] 非空，服务端随机数二进制字节数据
 * @param srvr_rand_len       [in] 非空，服务端随机数长度
 * @param srvr_sig_cert       [in] 可空，服务端签名证书DER编码的二进制字节数据
 * @param srvr_sig_cert_len   [in] 可空，服务端签名证书长度
 * @param srvr_enc_cert       [in] 可空，服务端加密证书DER编码的二进制字节数据
 * @param srvr_enc_cert_len   [in] 可空，服务端加密证书长度
 * @param srvr_sign           [in] 非空，服务端签名值DER编码的二进制数据
 * @param srvr_sign_len       [in] 非空，服务端签名值长度
 * @param pre_master_key      [out] 非空，预主密钥密文数据，建议缓冲区大小为[48+ISEC_MAX_KEYPAIR_LEN]字节
 * @param pre_master_key_len  [in] 非空，缓冲区大小 [out] 非空，预主密钥密文长度
 *
 * @return int                成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_client_key_exchange(isec_ke_t *ke,
    const unsigned char *srvr_rand, int srvr_rand_len,
    const unsigned char *srvr_sig_cert, int srvr_sig_cert_len,
    const unsigned char *srvr_enc_cert, int srvr_enc_cert_len,
    const unsigned char *srvr_sign, int srvr_sign_len,
    unsigned char *pre_master_key, int *pre_master_key_len);

/**
 * @brief 服务端密钥协商
 *
 * 服务端必选：
 *   从客户端接收预主密钥密文并调用本接口进行服务端密钥协商。
 * 流程：
 *   从客户端接收预主密钥密文；
 *   调用本接口进行密钥协商，可选输出协商的所有密钥的明文；
 *   服务端密钥协商完成，后续可使用isec_ke_encrypt/isec_ke_decrypt/isec_ke_hmac/isec_ke_cmac 进行算法调用；
 * 内部实现：
 *   使用加载的加密私钥解密传入的预主密钥密文；
 *   对【32字节客户端随机数+32字节服务端随机数+48字节预主密钥】进行KDF计算对称算法、HMAC、CMAC的密钥；
 *   若密钥出参非空，则导出密钥的明文；
 *   使用白盒算法加密对称算法、HMAC、CMAC的密钥并保存到句柄中，并从内存中清除明文的密钥；
 *
 * @param ke                  [in] 非空，密钥协商句柄
 * @param pre_master_key      [in] 非空，预主密钥密文数据
 * @param pre_master_key_len  [in] 非空，预主密钥密文长度
 * @param key                 [out] 可空，明文对称密钥（传NULL时，由key_len返回缓冲区大小，建议为[ISEC_MAX_KEY_LEN]字节）
 * @param key_len             [in] 可空，明文对称密钥缓冲区大小 [out] 可空，明文对称密钥长度
 * @param iv                  [out] 可空，初始化向量（传NULL时，由iv_len返回缓冲区大小，建议为[ISEC_MAX_IV_LEN]字节）
 * @param iv_len              [in] 可空，初始化向量缓冲区大小 [out] 可空，初始化向量长度
 * @param hmac_key            [out] 可空，HMAC密钥（传NULL时，由hmac_key_len返回缓冲区大小，建议为[ISEC_MAX_KEY_LEN]字节）
 * @param hmac_key_len        [in] 可空，HMAC密钥缓冲区
 * @param cmac_key            [out] 可空，CMAC密钥（传NULL时，由cmac_key_len返回缓冲区大小，建议为[ISEC_MAX_KEY_LEN]字节）
 * @param cmac_key_len        [in] 可空，CMAC密钥缓冲区大小 [out] 可空，CMAC密钥长度
 *
 * @return int                成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_server_key_exchange(isec_ke_t *ke, const void *pre_master_key, int pre_master_key_len,
    unsigned char *key, int *key_len, unsigned char *iv, int *iv_len,
    unsigned char *hmac_key, int *hmac_key_len, unsigned char *cmac_key, int *cmac_key_len);

/**
 * @brief 对称加密
 *
 * 通过协商出来的存在白盒中的密钥进行对称加密。
 *
 * 当前版本对称算法固定为SM4-CBC，使用PKCS5填充
 *
 * @param ke          [in] 非空，密钥协商句柄
 * @param data        [in] 非空，原文数据
 * @param data_len    [in] 非空，原文长度
 * @param cipher      [out] 可空，密文数据（传NULL时，由cipher_len返回缓冲区大小，建议为[data_len+ISEC_MAX_KEY_LEN]字节）
 * @param cipher_len  [in] 非空，密文缓冲区大小 [out] 非空，密文长度
 *
 * @return int        成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_encrypt(isec_ke_t *ke, const void *data, int data_len,
    unsigned char *cipher, int *cipher_len);

/**
 * @brief 对称解密
 *
 * 通过协商出来的存在白盒中的密钥进行对称解密。
 *
 * 当前版本对称算法固定为SM4-CBC，使用PKCS5填充
 *
 * @param ke          [in] 非空，密钥协商句柄
 * @param cipher      [in] 非空，密文数据
 * @param cipher_len  [in] 非空，密文长度
 * @param data        [in] 可空，原文数据（传NULL时，由data_len返回缓冲区大小，建议为[cipher_len]字节）
 * @param data_len    [in] 非空，原文缓冲区大小 [out] 非空，原文长度
 *
 * @return int        成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_decrypt(isec_ke_t *ke, const unsigned char *cipher, int cipher_len,
    void *data, int *data_len);

/**
 * @brief HMAC消息鉴别码
 *
 * 通过协商出来的存在白盒中的密钥进行HMAC消息鉴别码计算。
 *
 * 当前版本HMAC摘要算法固定为SM3
 *
 * @param ke        [in] 非空，密钥协商句柄
 * @param data      [in] 非空，原文数据
 * @param data_len  [in] 非空，原文长度
 * @param hmac      [in] 可空，消息鉴别码数据（传NULL时，由hmac_len返回缓冲区大小，建议为[ISEC_MAX_MD_LEN]字节）
 * @param hmac_len  [in] 非空，消息鉴别码缓冲区大小 [out] 非空，消息鉴别码长度
 *
 * @return int      成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_hmac(isec_ke_t *ke, const void *data, int data_len,
    unsigned char *hmac, int *hmac_len);

/**
 * @brief CMAC消息鉴别码
 *
 * 通过协商出来的存在白盒中的密钥进行CMAC消息鉴别码计算。
 *
 * 当前版本CMAC对称算法固定为SM4
 *
 * @param ke        [in] 非空，密钥协商句柄
 * @param data      [in] 非空，原文数据
 * @param data_len  [in] 非空，原文长度
 * @param cmac      [out] 可空，消息鉴别码数据（传NULL时，由cmac_len返回缓冲区大小，建议为[ISEC_MAX_MD_LEN]字节）
 * @param cmac_len  [in] 非空，消息鉴别码缓冲区大小 [out] 非空，消息鉴别码长度
 *
 * @return int      成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_ke_cmac(isec_ke_t *ke, const void *data, int data_len,
    unsigned char *cmac, int *cmac_len);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __ISEC_KE_H */

