#ifndef __ISEC_WBSM4_H
#define __ISEC_WBSM4_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "isec.h"

/**
 * @brief 白盒句柄
 *
 * 白盒算法不保存、不使用明文密钥，而是使用保存在句柄内的白盒密钥表，实现SM4加解密
 *
 * 注1：白盒密钥表加密效果等同于直接使用明文密钥加解密
 * 注2：白盒密钥表不可恢复为明文密钥
 */
typedef void isec_wbsm4_t;

/**
 * @brief 创建白盒句柄
 *
 * 创建白盒句柄，该句柄内的白盒密钥表为空。
 *
 * 注1：若不存在明文密钥或密钥表，则后续应该调用isec_wbsm4_gen_key随机产生密钥表，再用于加解密
 * 注2：若存在明文密钥，则后续调用isec_wbsm4_set_key将明文密钥转换白盒密钥表，再用于加解密
 * 注3：若存在密钥表，则不调用本函数，而是调用isec_wbsm4_unmarshal直接恢复为白盒句柄，再用于加解密
 *
 * @param wbsm4  [out] 白盒句柄
 * @return int 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_wbsm4_new(isec_wbsm4_t **wbsm4);

/**
 * @brief 销毁白盒句柄
 *
 * 销毁白盒句柄及内部数据
 *
 * @param wbsm4  [in] 白盒句柄
 * @return int 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_wbsm4_free(isec_wbsm4_t *wbsm4);

/**
 * @brief 备份白盒句柄
 *
 * 备份白盒句柄内部的白盒密钥表
 *
 * @param wbsm4      [in] 非空，白盒句柄
 * @param table      [out] 可空，密钥表备份数据
 * @param table_len  [in] 可空，密钥表备份缓冲区大小 [out] 可空，密钥表备份长度
 * @return int 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_wbsm4_marshal(isec_wbsm4_t *wbsm4, unsigned char *table, int *table_len);

/**
 * @brief 恢复白盒密钥表
 *
 * 恢复白盒密钥表为白盒句柄，该句柄可直接用于加解密
 *
 * @param table      [in] 非空，密钥表备份数据
 * @param table_len  [in] 正数，密钥表备份长度
 * @param wbsm4      [out] 非空，白盒句柄
 * @return int 成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_wbsm4_unmarshal(const unsigned char *table, int table_len, isec_wbsm4_t **wbsm4);

/**
 * @brief 产生白盒密钥表
 *
 * 随机产生白盒密钥表，用于SM4加解密
 *
 * @param wbsm4  [in] 非空，白盒句柄
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_wbsm4_gen_key(isec_wbsm4_t *wbsm4);

/**
 * @brief 设置白盒密钥
 *
 * 根据外部明文SM4密钥，产生白盒密钥表并保存在内部。
 *
 * @param wbsm4    [in] 非空，白盒句柄
 * @param key      [in] 非空，SM4对称密钥
 * @param key_len  [in] 非空，SM4对称密钥长度
 * @param iv       [in] 可空，SM4初始化向量（若加密模式使用ECB则该参数可空，其他模式非空）
 * @param iv_len   [in] 可空，SM4初始化向量长度（若加密模式使用ECB则该参数可为0，其他模式非0）
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_wbsm4_set_key(isec_wbsm4_t *wbsm4, const void *key, int key_len, const void *iv, int iv_len);

/**
 * @brief 白盒SM4加密
 *
 * 使用白盒SM4算法对数据进行加密
 *
 * @param wbsm4     [in] 白盒句柄
 * @param mode      [in] 加密算法模式，当前仅支持ECB和CBC模式
 * @param padding   [in] 填充模式
 * @param data      [in] 待加密数据
 * @param data_len  [in] 待加密数据长度
 * @param enc       [out] 密文数据，建议缓冲区大小为：待加密数据字节长度+ISEC_MAX_KEY_LEN
 * @param enc_len   [in] 密文缓冲区大小 [out] 密文长度
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_wbsm4_encrypt(isec_wbsm4_t *wbsm4, isec_cipher_mode_e mode, isec_cipher_padding_e padding,
    const void *data, int data_len, unsigned char *enc, int *enc_len);

/**
 * @brief 白盒SM4解密
 *
 * 使用白盒SM4算法对密文进行解密
 *
 * @param wbsm4    [in] 白盒密钥表
 * @param mode     [in] 加密算法模式，当前仅支持ECB和CBC模式
 * @param padding  [in] 填充模式
 * @param enc      [in] 密文数据
 * @param enc_len  [in] 密文长度
 * @param dec      [in] 解密数据，建议缓冲区大小为：密文长度
 * @param dec_len  [in] 解密数据缓冲区大小 [out] 解密数据长度
 * @return int  成功返回isec_err_none，其他返回错误码
 */
ISEC_API
int isec_wbsm4_decrypt(isec_wbsm4_t *wbsm4, isec_cipher_mode_e mode, isec_cipher_padding_e padding,
    const unsigned char *enc, int enc_len, unsigned char *dec, int *dec_len);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __ISEC_WBSM4_H */

